# from percentage import Percentage
from enum import Enum
from functools import wraps

from percentage import percentage


class Color(Enum):
    RED = "\x1b[1;31m",
    GREEN = "\x1b[1;32m",
    GRAY = "\x1b[1;37m",
    NONE = "\033[0m",


def cartridge_return(func):
    @wraps(func)
    def returned(*args, **kwargs):
        print("\r", end="")
        result = func(*args, **kwargs)
        return result

    return returned


def color_print(color=Color.NONE):
    def colored(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return str(color.value[0]) + str(func(*args, **kwargs))
        return wrapper
    return colored


class BarPercentage(percentage.Percentage):
    def __init__(self, mimnum=0, maximum=100):
        super().__init__(mimnum, maximum)
        self.shape = "━"
        self.stage = 50

    @cartridge_return
    def update(self):
        pass_blocks = int(self.value.get_level() / self.maximum * self.stage)
        result = self.get_perfect_bar() if pass_blocks == self.stage else self.get_not_perfect_bar(pass_blocks)
        print(result, end="")
        self.print_captions()

    @color_print(Color.GREEN)
    def get_perfect_bar(self):
        return self.shape * self.stage

    def get_not_perfect_bar(self, pass_blocks):
        return self.get_cleared_bar(pass_blocks) + self.get_not_cleared_bar(pass_blocks)

    @color_print(Color.RED)
    def get_cleared_bar(self, pass_blocks):
        return self.shape * pass_blocks

    @color_print(Color.GRAY)
    def get_not_cleared_bar(self, pass_blocks):
        return self.shape * (self.stage - pass_blocks)

    def print_captions(self):
        print("", end=" ")
        for cap in self.captions:
            print(self.get_caption(cap), end=" ")

    @color_print(Color.NONE)
    def get_caption(self, caption):
        return caption.process()



from functools import partial


class BuilderPattern:
    def __init__(self, cls, build_method_name, setter_prefix, type_check):
        temp_class = cls()
        self.__dict__.update(
            {str(setter_prefix) + key: partial(self.__set_variable, key) for key in temp_class.__dict__})
        if type_check:
            self.__type = {key: type(key) for key in temp_class.__dict__}
        del temp_class
        setattr(self, build_method_name, self.__build)
        self.type_check = type_check
        self.__parameters = {}
        self.__main_class = cls

    def __set_variable(self, key, value):
        if self.type_check: assert not isinstance(value, self.__type[key]), TypeError
        self.__parameters.update({key: value})
        return self

    def __build(self):
        main = self.__main_class()
        main.__dict__.update(self.__parameters)
        return main

    def __str__(self):
        return f"{self.__main_class.__name__} builder"

    def __repr__(self):
        return f"{self.__main_class.__name__} builder, {self.__parameters}"


def builder(build_method_name="build", setter_prefix="", type_check=False):
    def decorator(cls):
        setattr(cls, 'builder', partial(BuilderPattern, cls, build_method_name, setter_prefix, type_check))
        return cls

    return decorator



from abc import ABC
from abc import abstractmethod
from functools import wraps


class Percentage(ABC):
    def __init__(self, minimum=0, maximum=100):
        self.minimum = minimum
        self.maximum = maximum
        self.value = MinMaxLevelNumber(self.minimum, self.maximum)
        self.captions = []

    def add_caption(self, caption):
        assert issubclass(caption, PercentageCaption), ValueError
        cap = caption()
        self.captions.append(cap.register(self))

    def add_value(self, value):
        self.value.set_value(self.value.get_level() + value)

    def clear(self):
        self.value.set_value(self.minimum)
        for cap in self.captions:
            cap.register(self)

    @abstractmethod
    def update(self):
        raise NotImplementedError

    def get_value(self):
        return self.value.get_level()

    def get_minimum(self):
        return self.minimum

    def get_maximum(self):
        return self.maximum


class PercentageCaption(ABC):
    @abstractmethod
    def register(self, percentage: Percentage) -> 'PercentageCaption':
        raise NotImplementedError

    @abstractmethod
    def process(self) -> str:
        raise NotImplementedError


def min_max_setting(func):
    @wraps(func)
    def inner(*args):
        self, min, max = args
        assert min < max, ValueError
        self.__dict__["min"] = min
        self.__dict__["max"] = max
        self.__dict__["level"] = min

    return inner


class MinMaxLevelNumber:
    @min_max_setting
    def __init__(self, min, max):
        ...

    def set_value(self, value):
        self.level = value

    def get_level(self):
        return self.level

    def __setattr__(self, key, value):
        assert self.min <= value <= self.max, ValueError
        self.__dict__[key] = value

    def __repr__(self):
        return f"{self.level}"



import time

from percentage import Percentage, PercentageCaption


class PercentageTinyTimeSpanCaption(PercentageCaption):
    def __init__(self):
        self.start = time.time()

    def register(self, percentage: Percentage) -> PercentageCaption:
        self.start = time.time()
        return self

    def process(self, round_n: int = 2) -> str:
        return f"{round(time.time() - self.start, round_n)} s"


class PercentageValueCaption(PercentageCaption):
    def __init__(self):
        self.percentage = None

    def register(self, percentage: Percentage) -> PercentageCaption:
        self.percentage = percentage
        return self

    def process(self) -> str:
        maximum = self.percentage.get_maximum()
        value = self.percentage.get_value()
        return f"[{value}/{maximum}]"



from percentage import percentage
# import percentage.captions

Percentage = percentage
MinMaxLevelNumber = percentage.MinMaxLevelNumber
PercentageCaption = percentage.PercentageCaption

# PercentageValueCaption = captions.PercentageValueCaption
# PercentageTinyTimeSpanCaption = captions.PercentageTinyTimeSpanCaption



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyasset", # Replace with your own username
    version="1.0.1",
    author="minwoo lee",
    author_email="dlalsdn1009@naver.com",
    description="Python Asset Library.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)